<?php

namespace Drupal\entity_mesh\Controller;

use Drupal\Core\Database\Query\Condition;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for Entity Mesh note routes.
 */
class EntityMeshNodeController extends EntityMeshController {

  /**
   * Entity Id.
   *
   * @var int
   */
  protected int $entityId;

  /**
   * Entity Type.
   *
   * @var string
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  public function query() {

    $query = parent::query();

    $db_and = new Condition('AND');
    $db_and->condition('em.source_entity_type', $this->entityType);
    $db_and->condition('em.source_entity_id', (string) $this->entityId);

    $db_and2 = new Condition('AND');
    $db_and2->condition('em.target_entity_type', $this->entityType);
    $db_and2->condition('em.target_entity_id', (string) $this->entityId);

    $db_or = new Condition('OR');
    $db_or->condition($db_and);
    $db_or->condition($db_and2);
    $query->condition($db_or);

    return $query;
  }

  /**
   * Builds the response.
   */
  public function build($node = NULL) {

    if (!is_numeric($node)) {
      throw new NotFoundHttpException();
    }

    $this->entityId = (int) $node;
    $this->entityType = 'node';

    $build = parent::build();

    // @todo Dependency injection!
    $links_header = [
      $this->t('Href'),
      $this->t('Link Type'),
      $this->t('Entity Type'),
      $this->t('Entity Id'),
      $this->t('Entity Label'),
      $this->t('Link'),
    ];

    $links_table = [];
    $backlinks_table = [];

    foreach ($this->results() as $result) {
      if (!isset($result->target_href)) {
        continue;
      }

      // Source values:
      $source_entity_type = $result->source_entity_type ?? NULL;
      $source_entity_id = $result->source_entity_id ?? NULL;
      $source_type = $result->source_type ?? NULL;
      $source_title = $result->source_title ?? NULL;

      // Target values:
      $target_href = $result->target_href ?? NULL;
      $target_type = $result->target_type ?? NULL;
      $target_entity_type = $result->target_entity_type ?? NULL;
      $target_entity_id = $result->target_entity_id ?? NULL;
      $target_title = $result->target_title ?? NULL;

      $row = [];

      if ($source_entity_type == 'node' && $source_entity_id == $node) {
        // Validate Object properties:
        $row[] = $target_href;
        $row[] = $result->target_link_type ?? NULL;
        $row[] = $target_entity_type . (!empty($target_type) ? ' - ' . $target_type : '');
        $row[] = $target_entity_id;
        $row[] = $target_title;

        if (($result->target_link_type ?? NULL) == 'internal' &&
          strpos($target_entity_type, 'broken-link') === FALSE) {
          $target_path = $result->target_path ?? '';
          $url = Url::fromUri('internal:' . $target_path);
          $url->setOption('attributes', ['target' => '_blank']);
          $row[] = Link::fromTextAndUrl($this->t('View'), $url);
        }

        $links_table[] = $row;
      }
      else {
        $row[] = $source_entity_type . ' - ' . $source_type;
        $row[] = $source_entity_id;
        $row[] = $source_title;

        $url = Url::fromRoute('entity.' . $source_entity_type . '.canonical', [$source_entity_type => $source_entity_id]);
        if ($url->access()) {
          $url->setOption('attributes', ['target' => '_blank']);
          $row[] = Link::fromTextAndUrl($this->t('View'), $url);
        }

        $backlinks_table[] = $row;
      }
    }

    // Links:
    if (!empty($links_table)) {
      $build['links-header'] = [
        '#type' => 'item',
        '#markup' => '<h2>' . $this->t('Links') . '</h2>',
      ];
      $build['links'] = [
        '#type' => 'table',
        '#header' => $links_header,
        '#rows' => $links_table,
      ];
    }

    // Links:
    $build['backlinks-header'] = [
      '#type' => 'item',
      '#markup' => '<h2>' . $this->t('Backlinks') . '</h2>',
    ];

    // Backlinks:
    $backlinks_header = [
      $this->t('Entity Type'),
      $this->t('Entity Id'),
      $this->t('Entity Label'),
      $this->t('Link'),
    ];

    if (!empty($backlinks_table)) {
      $build['backlinks'] = [
        '#type' => 'table',
        '#header' => $backlinks_header,
        '#rows' => $backlinks_table,
      ];
    }

    return $build;
  }

}
